% data = imdata(img, permuteDims)
%
% imdata converts an otb image object to a matlab array
%
% This is part of the OTB Matlab Toolbox
% 
% author: Rik Bellens

function data = imdata(img, permuteDims)

if nargin<2, permuteDims = true; end

sx = suffix(img);
if sx(1)=='V'
    pixelType = sx(3:(end-1));
else
    pixelType = sx(2:(end-1));
end

switch pixelType
    case {'D'}
        matClass = 'double';
        ptrMethod = 'long2DoublePtr';
        copyMethod = 'copyDArray';
    case {'F'}
        matClass = 'single';
        ptrMethod = 'long2FloatPtr';
        copyMethod = 'copyFArray';
    case {'UC'}
        data = uint8(imdata(castimage(img,'UL'),permuteDims));
        return;
    case {'US'}
        data = uint16(imdata(castimage(img,'UL'),permuteDims));
        return;
    case {'UL'}
        matClass = 'uint32';
        ptrMethod ='long2ULongPtr';
        copyMethod = 'copyULArray';
    case {'C'}
        data = int8(imdata(castimage(img,'L'),permuteDims));
        return;
    case {'S'}
        data = int16(imdata(castimage(img,'L'),permuteDims));
        return;
    case {'L'}
        matClass = 'int32';
        ptrMethod = 'long2LongPtr';
        copyMethod = 'copyLArray';
end

ncomp = img.Call('GetNumberOfComponentsPerPixel');
dim = img.Call('GetImageDimension');
region = img.Call('GetBufferedRegion');
s = zeros(1,dim);
for i=1:dim
    s(i) = region.GetSize(i-1);
end
s = [ncomp,s];

data = zeros(s,matClass);
dst = javaMethod(ptrMethod,'InsightToolkit.MatlabExtras',getptr(data));
src = img.Call('GetBufferPointer');
javaMethod('copy',['InsightToolkit.vnl_c_vector' pixelType],src,dst,prod(s));

if permuteDims
    data = permute(data,[3 2 4:length(s) 1]);
end
