% img = otbimage(data, permuteDims)
%
% otbimage converts a matlab array to an otb image object
%
% This is part of the OTB Matlab Toolbox
% 
% author: Rik Bellens

function img = otbimage(data, permuteDims)

if nargin<2, permuteDims = true; end

matClass = class(data);
switch matClass
    case {'double'}
        sx = 'D';
        ptrMethod = 'long2DoublePtr';
        copyMethod = 'copyDArray';
    case {'single'}
        sx = 'F';
        ptrMethod = 'long2FloatPtr';
        copyMethod = 'copyFArray';
    case {'uint8'}
        img = castimage(otbimage(uint32(data),permuteDims),'UC');
        return
    case {'uint16'}
        img = castimage(otbimage(uint32(data),permuteDims),'US');
        return
    case {'uint32'}
        sx = 'UL';
        ptrMethod = 'long2ULongPtr';
        copyMethod = 'copyULArray';
    case {'int8'}
        img = castimage(otbimage(uint32(data),permuteDims),'C');
        return
    case {'int16'}
        img = castimage(otbimage(uint32(data),permuteDims),'S');
        return
    case {'int32'}
        sx = 'L';
        ptrMethod = 'long2LongPtr';
        copyMethod = 'copyLArray';
end

s = size(data);
if length(s)==2, s = [s 1]; end
if permuteDims
    data = permute(data,[length(s) 2 1 3:(length(s)-1)]);
end
s = size(data);
ncomp = s(1);
ndim = length(s)-1;
s = s(2:end);

PixelType = sx;
if ncomp==1
    sx = ['Image' sx int2str(ndim)];
else
    sx = ['VectorImage' sx int2str(ndim)];
end

img = VerySmartPointer(['otb' sx]);
img.Call('SetNumberOfComponentsPerPixel',ncomp);
region = javaObject(['InsightToolkit.itkImageRegion' int2str(ndim)]);
for i=1:ndim
    region.SetSize(i-1,s(i));
end
img.Call('SetRegions',region);
img.Call('Allocate');
dst = img.Call('GetBufferPointer');
src = javaMethod(ptrMethod,'InsightToolkit.MatlabExtras',getptr(data));
javaMethod('copy',['InsightToolkit.vnl_c_vector' PixelType],src,dst,prod(s)*ncomp);

